@@ -16,7 +16,6 @@
  * Boston, MA 021110-1307, USA.
  */
 
-
 #include "kerncompat.h"
 
 #include <ctype.h>
@@ -26,10 +25,12 @@
 #include <fcntl.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#if COMPRESSION_LZO
 #include <lzo/lzoconf.h>
 #include <lzo/lzo1x.h>
+#endif
 #include <zlib.h>
-#if BTRFSRESTORE_ZSTD
+#if COMPRESSION_ZSTD
 #include <zstd.h>
 #endif
 #include <regex.h>
@@ -99,6 +100,10 @@ static inline size_t read_compress_length(unsigned char *buf)
 static int decompress_lzo(struct btrfs_root *root, unsigned char *inbuf,
 			char *outbuf, u64 compress_len, u64 *decompress_len)
 {
+#if !COMPRESSION_LZO
+	error("btrfs-restore not compiled with lzo support");
+	return -1;
+#else
 	size_t new_len;
 	size_t in_len;
 	size_t out_len = 0;
@@ -157,12 +162,13 @@ static int decompress_lzo(struct btrfs_root *root, unsigned char *inbuf,
 	*decompress_len = out_len;
 
 	return 0;
+#endif
 }
 
 static int decompress_zstd(const char *inbuf, char *outbuf, u64 compress_len,
 			   u64 decompress_len)
 {
-#if !BTRFSRESTORE_ZSTD
+#if !COMPRESSION_ZSTD
 	error("btrfs not compiled with zstd support");
 	return -1;
 #else
@@ -303,15 +309,15 @@ static int copy_one_inline(struct btrfs_root *root, int fd,
 			    struct btrfs_file_extent_item);
 	ptr = btrfs_file_extent_inline_start(fi);
 	len = btrfs_file_extent_ram_bytes(leaf, fi);
-	inline_item_len = btrfs_file_extent_inline_item_len(leaf, btrfs_item_nr(path->slots[0]));
+	inline_item_len = btrfs_file_extent_inline_item_len(leaf, path->slots[0]);
 	read_extent_buffer(leaf, buf, ptr, inline_item_len);
 
 	compress = btrfs_file_extent_compression(leaf, fi);
 	if (compress == BTRFS_COMPRESS_NONE) {
 		done = pwrite(fd, buf, len, pos);
 		if (done < len) {
-			fprintf(stderr, "Short inline write, wanted %d, did "
-				"%zd: %d\n", len, done, errno);
+			error("short inline write, wanted %d, did %zd: %m",
+					len, done);
 			return -1;
 		}
 		return 0;
@@ -334,8 +340,8 @@ static int copy_one_inline(struct btrfs_root *root, int fd,
 	done = pwrite(fd, outbuf, ram_size, pos);
 	free(outbuf);
 	if (done < ram_size) {
-		fprintf(stderr, "Short compressed inline write, wanted %Lu, "
-			"did %zd: %d\n", ram_size, done, errno);
+		error("short compressed inline write, wanted %llu, did %zd: %m",
+				ram_size, done);
 		return -1;
 	}
 
@@ -408,8 +414,8 @@ again:
 	cur = bytenr;
 	while (cur < bytenr + size_left) {
 		length = bytenr + size_left - cur;
-		ret = read_extent_data(root->fs_info, inbuf + cur - bytenr, cur,
-				       &length, mirror_num);
+		ret = read_data_from_disk(root->fs_info, inbuf + cur - bytenr, cur,
+					  &length, mirror_num);
 		if (ret < 0) {
 			mirror_num++;
 			if (mirror_num > num_copies) {
@@ -467,34 +473,6 @@ out:
 	return ret;
 }
 
-enum loop_response {
-	LOOP_STOP,
-	LOOP_CONTINUE,
-	LOOP_DONTASK
-};
-
-static enum loop_response ask_to_continue(const char *file)
-{
-	char buf[16];
-	char *ret;
-
-	printf("We seem to be looping a lot on %s, do you want to keep going "
-	       "on ? (y/N/a): ", file);
-again:
-	ret = fgets(buf, 16, stdin);
-	if (!ret || *ret == '\n' || tolower(*ret) == 'n')
-		return LOOP_STOP;
-	if (tolower(*ret) == 'a')
-		return LOOP_DONTASK;
-	if (tolower(*ret) != 'y') {
-		printf("Please enter one of 'y', 'n', or 'a': ");
-		goto again;
-	}
-
-	return LOOP_CONTINUE;
-}
-
-
 static int set_file_xattrs(struct btrfs_root *root, u64 inode,
 			   int fd, const char *file_name)
 {
@@ -541,7 +519,7 @@ static int set_file_xattrs(struct btrfs_root *root, u64 inode,
 		if (key.type != BTRFS_XATTR_ITEM_KEY || key.objectid != inode)
 			break;
 		cur = 0;
-		total_len = btrfs_item_size_nr(leaf, path.slots[0]);
+		total_len = btrfs_item_size(leaf, path.slots[0]);
 		di = btrfs_item_ptr(leaf, path.slots[0],
 				    struct btrfs_dir_item);
 
@@ -653,7 +631,6 @@ static int copy_file(struct btrfs_root *root, int fd, struct btrfs_key *key,
 	int ret;
 	int extent_type;
 	int compression;
-	int loops = 0;
 	u64 found_size = 0;
 	struct timespec times[2];
 	int times_ok = 0;
@@ -716,22 +693,11 @@ static int copy_file(struct btrfs_root *root, int fd, struct btrfs_key *key,
 	}
 
 	while (1) {
-		if (loops >= 0 && loops++ >= 1024) {
-			enum loop_response resp;
-
-			resp = ask_to_continue(file);
-			if (resp == LOOP_STOP)
-				break;
-			else if (resp == LOOP_CONTINUE)
-				loops = 0;
-			else if (resp == LOOP_DONTASK)
-				loops = -1;
-		}
 		if (path.slots[0] >= btrfs_header_nritems(leaf)) {
 			do {
 				ret = next_leaf(root, &path);
 				if (ret < 0) {
-					fprintf(stderr, "Error searching %d\n", ret);
+					error("search to next leaf failed: %d", ret);
 					goto out;
 				} else if (ret) {
 					/* No more leaves to search */
@@ -853,8 +819,7 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 	if (ret == 2) {
 		ret = unlink(path_name);
 		if (ret) {
-			fprintf(stderr, "failed to unlink '%s' for overwrite\n",
-					path_name);
+			error("failed to unlink '%s' for overwrite: %m", path_name);
 			return ret;
 		}
 	}
@@ -868,7 +833,7 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 
 	leaf = path.nodes[0];
 	if (!leaf) {
-		fprintf(stderr, "Error getting leaf for symlink '%s'\n", file);
+		error("failed to get leaf for symlink '%s'", file);
 		ret = -1;
 		goto out;
 	}
@@ -876,10 +841,9 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 	extent_item = btrfs_item_ptr(leaf, path.slots[0],
 			struct btrfs_file_extent_item);
 
-	len = btrfs_file_extent_inline_item_len(leaf,
-			btrfs_item_nr(path.slots[0]));
+	len = btrfs_file_extent_inline_item_len(leaf, path.slots[0]);
 	if (len >= PATH_MAX) {
-		fprintf(stderr, "Symlink '%s' target length %d is longer than PATH_MAX\n",
+		error("symlink '%s' target length %d is longer than PATH_MAX",
 				fs_name, len);
 		ret = -1;
 		goto out;
@@ -894,8 +858,7 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 	if (!dry_run) {
 		ret = symlink(symlink_target, path_name);
 		if (ret < 0) {
-			fprintf(stderr, "Failed to restore symlink '%s': %m\n",
-					path_name);
+			error("failed to restore symlink '%s': %m", path_name);
 			goto out;
 		}
 	}
@@ -918,7 +881,7 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 
 	ret = btrfs_lookup_inode(NULL, root, &path, key, 0);
 	if (ret) {
-		fprintf(stderr, "Failed to lookup inode for '%s'\n", file);
+		error("failed to lookup inode for '%s'", file);
 		goto out;
 	}
 
@@ -929,7 +892,7 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 				   btrfs_inode_gid(path.nodes[0], inode_item),
 				   AT_SYMLINK_NOFOLLOW);
 	if (ret) {
-		fprintf(stderr, "Failed to change owner: %m\n");
+		error("failed to change owner of '%s': %m", file);
 		goto out;
 	}
 
@@ -943,7 +906,7 @@ static int copy_symlink(struct btrfs_root *root, struct btrfs_key *key,
 
 	ret = utimensat(AT_FDCWD, file, times, AT_SYMLINK_NOFOLLOW);
 	if (ret)
-		fprintf(stderr, "Failed to set times: %m\n");
+		error("failed to set times for '%s': %m", file);
 out:
 	btrfs_release_path(&path);
 	return ret;
@@ -962,7 +925,6 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 	int name_len;
 	int ret = 0;
 	int fd;
-	int loops = 0;
 	u8 type;
 
 	btrfs_init_path(&path);
@@ -970,7 +932,7 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 	key->type = BTRFS_DIR_INDEX_KEY;
 	ret = btrfs_search_slot(NULL, root, key, &path, 0, 0);
 	if (ret < 0) {
-		fprintf(stderr, "Error searching %d\n", ret);
+		error("search for next directory entry failed: %d", ret);
 		goto out;
 	}
 
@@ -982,8 +944,7 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 			   "No leaf after search, looking for the next leaf\n");
 		ret = next_leaf(root, &path);
 		if (ret < 0) {
-			fprintf(stderr, "Error getting next leaf %d\n",
-				ret);
+			error("search for next leaf failed: %d", ret);
 			goto out;
 		} else if (ret > 0) {
 			/* No more leaves to search */
@@ -996,19 +957,11 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 	}
 
 	while (leaf) {
-		if (loops++ >= 1024) {
-			printf("We have looped trying to restore files in %s "
-			       "too many times to be making progress, "
-			       "stopping\n", in_dir);
-			break;
-		}
-
 		if (path.slots[0] >= btrfs_header_nritems(leaf)) {
 			do {
 				ret = next_leaf(root, &path);
 				if (ret < 0) {
-					fprintf(stderr, "Error searching %d\n",
-						ret);
+					error("search for next leaf failed: %d", ret);
 					goto out;
 				} else if (ret > 0) {
 					/* No more leaves to search */
@@ -1023,7 +976,7 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 		}
 		btrfs_item_key_to_cpu(leaf, &found_key, path.slots[0]);
 		if (found_key.objectid != key->objectid) {
-			pr_verbose(2, "Found objectid=%Lu, key=%llu\n",
+			pr_verbose(2, "Found objectid=%llu, key=%llu\n",
 				   found_key.objectid, key->objectid);
 			break;
 		}
@@ -1062,19 +1015,16 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 				goto next;
 			fd = open(path_name, O_CREAT|O_WRONLY, 0644);
 			if (fd < 0) {
-				fprintf(stderr, "Error creating %s: %d\n",
-					path_name, errno);
+				error("creating '%s' failed: %m", path_name);
 				if (ignore_errors)
 					goto next;
 				ret = -1;
 				goto out;
 			}
-			loops = 0;
 			ret = copy_file(root, fd, &location, path_name);
 			close(fd);
 			if (ret) {
-				fprintf(stderr, "Error copying data for %s\n",
-					path_name);
+				error("copying data for %s failed", path_name);
 				if (ignore_errors)
 					goto next;
 				goto out;
@@ -1084,7 +1034,7 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 			char *dir = strdup(fs_name);
 
 			if (!dir) {
-				fprintf(stderr, "Ran out of memory\n");
+				error("ran out of memory");
 				ret = -ENOMEM;
 				goto out;
 			}
@@ -1105,10 +1055,8 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 								 &location);
 				if (IS_ERR(search_root)) {
 					free(dir);
-					fprintf(stderr, "Error reading "
-						"subvolume %s: %lu\n",
-						path_name,
-						PTR_ERR(search_root));
+					error("reading subvolume %s failed: %lu",
+						path_name, PTR_ERR(search_root));
 					if (ignore_errors)
 						goto next;
 					ret = PTR_ERR(search_root);
@@ -1122,8 +1070,7 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 				if (search_root->root_key.offset != 0 &&
 				    get_snaps == 0) {
 					free(dir);
-					printf("Skipping snapshot %s\n",
-					       filename);
+					printf("Skipping snapshot %s\n", filename);
 					goto next;
 				}
 				location.objectid = BTRFS_FIRST_FREE_OBJECTID;
@@ -1138,20 +1085,18 @@ static int search_dir(struct btrfs_root *root, struct btrfs_key *key,
 				ret = mkdir(path_name, 0755);
 			if (ret && errno != EEXIST) {
 				free(dir);
-				fprintf(stderr, "Error mkdiring %s: %d\n",
-					path_name, errno);
+				error("failed mkdir %s: %m", path_name);
 				if (ignore_errors)
 					goto next;
 				ret = -1;
 				goto out;
 			}
-			loops = 0;
 			ret = search_dir(search_root, &location,
 					 output_rootdir, dir, mreg);
 			free(dir);
 			if (ret) {
-				fprintf(stderr, "Error searching %s\n",
-					path_name);
+				error("searching directory %s failed: %d",
+					path_name, ret);
 				if (ignore_errors)
 					goto next;
 				goto out;
@@ -1174,7 +1119,7 @@ next:
 		snprintf(path_name, PATH_MAX, "%s%s", output_rootdir, in_dir);
 		fd = open(path_name, O_RDONLY);
 		if (fd < 0) {
-			fprintf(stderr, "ERROR: Failed to access %s to restore metadata\n",
+			error("failed to access '%s' to restore metadata: %m",
 					path_name);
 			if (!ignore_errors) {
 				ret = -1;
@@ -1218,7 +1163,7 @@ static int do_list_roots(struct btrfs_root *root)
 	key.type = BTRFS_ROOT_ITEM_KEY;
 	ret = btrfs_search_slot(NULL, root, &key, &path, 0, 0);
 	if (ret < 0) {
-		fprintf(stderr, "Failed to do search %d\n", ret);
+		error("failed search next root item: %d", ret);
 		btrfs_release_path(&path);
 		return -1;
 	}
@@ -1245,7 +1190,7 @@ static int do_list_roots(struct btrfs_root *root)
 		read_extent_buffer(leaf, &ri, offset, sizeof(ri));
 		printf(" tree ");
 		btrfs_print_key(&disk_key);
-		printf(" %Lu level %d\n", btrfs_root_bytenr(&ri),
+		printf(" %llu level %d\n", btrfs_root_bytenr(&ri),
 		       btrfs_root_level(&ri));
 		path.slots[0]++;
 	}
@@ -1274,7 +1219,8 @@ static struct btrfs_root *open_fs(const char *dev, u64 root_location,
 		ocf.filename = dev;
 		ocf.sb_bytenr = bytenr;
 		ocf.root_tree_bytenr = root_location;
-		ocf.flags = OPEN_CTREE_PARTIAL | OPEN_CTREE_NO_BLOCK_GROUPS;
+		ocf.flags = OPEN_CTREE_PARTIAL | OPEN_CTREE_NO_BLOCK_GROUPS |
+			    OPEN_CTREE_ALLOW_TRANSID_MISMATCH;
 		fs_info = open_ctree_fs_info(&ocf);
 		if (fs_info)
 			break;
@@ -1299,7 +1245,7 @@ static struct btrfs_root *open_fs(const char *dev, u64 root_location,
 		root->node = read_tree_block(fs_info, root_location,
 					     generation);
 		if (!extent_buffer_uptodate(root->node)) {
-			fprintf(stderr, "Error opening tree root\n");
+			error("opening tree root failed");
 			close_ctree(root);
 			return NULL;
 		}
@@ -1313,8 +1259,7 @@ static struct btrfs_root *open_fs(const char *dev, u64 root_location,
 		key.offset = (u64)-1;
 		fs_info->fs_root = btrfs_read_fs_root_no_cache(fs_info, &key);
 		if (IS_ERR(fs_info->fs_root)) {
-			fprintf(stderr, "Couldn't read fs root: %ld\n",
-				PTR_ERR(fs_info->fs_root));
+			error("could not read fs root: %ld", PTR_ERR(fs_info->fs_root));
 			close_ctree(fs_info->tree_root);
 			return NULL;
 		}
@@ -1342,12 +1287,12 @@ static int find_first_dir(struct btrfs_root *root, u64 *objectid)
 	key.offset = 0;
 	ret = btrfs_search_slot(NULL, root, &key, &path, 0, 0);
 	if (ret < 0) {
-		fprintf(stderr, "Error searching %d\n", ret);
+		error("searching next directory entry failed: %d", ret);
 		goto out;
 	}
 
 	if (!path.nodes[0]) {
-		fprintf(stderr, "No leaf!\n");
+		error("no leaf when looking for directory");
 		goto out;
 	}
 again:
@@ -1357,7 +1302,7 @@ again:
 		if (found_key.type != key.type)
 			continue;
 
-		printf("Using objectid %Lu for first dir\n",
+		printf("Using objectid %llu for first dir\n",
 		       found_key.objectid);
 		*objectid = found_key.objectid;
 		ret = 0;
@@ -1366,11 +1311,10 @@ again:
 	do {
 		ret = next_leaf(root, &path);
 		if (ret < 0) {
-			fprintf(stderr, "Error getting next leaf %d\n",
-				ret);
+			error("search for next leaf failed: %d", ret);
 			goto out;
 		} else if (ret > 0) {
-			fprintf(stderr, "No more leaves\n");
+			error("no more leaves to search");
 			goto out;
 		}
 	} while (!path.nodes[0]);
@@ -1383,30 +1327,46 @@ out:
 }
 
 static const char * const cmd_restore_usage[] = {
-	"btrfs restore [options] <device> <path> | -l <device>",
+	"btrfs restore [options] <device> <path>\n"
+	"btrfs restore [options] -l <device>",
 	"Try to restore files from a damaged filesystem (unmounted)",
 	"",
-	"-s|--snapshots       get snapshots",
-	"-x|--xattr           restore extended attributes",
-	"-m|--metadata        restore owner, mode and times",
-	"-S|--symlink         restore symbolic links",
-	"-i|--ignore-errors   ignore errors",
-	"-o|--overwrite       overwrite",
-	"-t <bytenr>          tree location",
-	"-f <bytenr>          filesystem location",
-	"-u|--super <mirror>  super mirror",
-	"-r|--root <rootid>   root objectid",
-	"-d                   find dir",
-	"-l|--list-roots      list tree roots",
-	"-D|--dry-run         dry run (only list files that would be recovered)",
-	"--path-regex <regex>",
-	"                     restore only filenames matching regex,",
-	"                     you have to use following syntax (possibly quoted):",
-	"                     ^/(|home(|/username(|/Desktop(|/.*))))$",
-	"-c                   ignore case (--path-regex only)",
-	"-v|--verbose         deprecated, alias for global -v option",
+	"  control:",
+	"       -D|--dry-run         dry run (only list files that would be recovered)",
+	"       -i|--ignore-errors   ignore errors",
+	"       -o|--overwrite       overwrite",
+	"  restoration:",
+	"       -m|--metadata        restore owner, mode and times",
+	"       -S|--symlink         restore symbolic links",
+	"       -s|--snapshots       get snapshots",
+	"       -x|--xattr           restore extended attributes",
+	"  filtering:",
+	"       --path-regex <regex>",
+	"                            restore only filenames matching regex,",
+	"                            you have to use following syntax (possibly quoted):",
+	"                            ^/(|home(|/username(|/Desktop(|/.*))))$",
+	"       -c                   ignore case (--path-regex only)",
+	"  analysis:",
+	"       -d                   find dir",
+	"       -l|--list-roots      list tree roots",
+	"  alternate starting point:",
+	"       -f <bytenr>          filesystem location",
+	"       -r|--root <rootid>   root objectid",
+	"       -t <bytenr>          tree location",
+	"       -u|--super <mirror>  super mirror",
+	"  other:",
+	"       -v|--verbose         deprecated, alias for global -v option",
 	HELPINFO_INSERT_GLOBALS,
 	HELPINFO_INSERT_VERBOSE,
+	"",
+	"Compression support: zlib"
+#if COMPRESSION_LZO
+		", lzo"
+#endif
+#if COMPRESSION_ZSTD
+		", zstd"
+#endif
+	,
 	NULL
 };
 
@@ -1431,7 +1391,7 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 	optind = 0;
 	while (1) {
 		int opt;
-		enum { GETOPT_VAL_PATH_REGEX = 256 };
+		enum { GETOPT_VAL_PATH_REGEX = GETOPT_VAL_FIRST };
 		static const struct option long_options[] = {
 			{ "path-regex", required_argument, NULL,
 				GETOPT_VAL_PATH_REGEX },
@@ -1476,8 +1436,8 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 			case 'u':
 				super_mirror = arg_strtou64(optarg);
 				if (super_mirror >= BTRFS_SUPER_MIRROR_MAX) {
-					fprintf(stderr, "Super mirror not "
-						"valid\n");
+					error("super mirror %d not valid",
+							super_mirror);
 					exit(1);
 				}
 				break;
@@ -1487,7 +1447,7 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 			case 'r':
 				root_objectid = arg_strtou64(optarg);
 				if (!is_fstree(root_objectid)) {
-					fprintf(stderr, "objectid %llu is not a valid fs/file tree\n",
+					error("objectid %llu is not a valid fs/file tree",
 							root_objectid);
 					exit(1);
 				}
@@ -1524,16 +1484,16 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 		usage(cmd);
 
 	if (fs_location && root_objectid) {
-		fprintf(stderr, "don't use -f and -r at the same time.\n");
+		error("can't use -f and -r at the same time");
 		return 1;
 	}
 
 	if ((ret = check_mounted(argv[optind])) < 0) {
 		errno = -ret;
-		fprintf(stderr, "Could not check mount status: %m\n");
+		error("could not check mount status: %m");
 		return 1;
 	} else if (ret) {
-		fprintf(stderr, "%s is currently mounted.  Aborting.\n", argv[optind]);
+		error("%s is currently mounted, cannot continue", argv[optind]);
 		return 1;
 	}
 
@@ -1548,7 +1508,7 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 		free_extent_buffer(root->node);
 		root->node = read_tree_block(root->fs_info, fs_location, 0);
 		if (!extent_buffer_uptodate(root->node)) {
-			fprintf(stderr, "Failed to read fs location\n");
+			error("failed to read fs location");
 			ret = 1;
 			goto out;
 		}
@@ -1557,7 +1517,7 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 	memset(path_name, 0, PATH_MAX);
 
 	if (strlen(argv[optind + 1]) >= PATH_MAX) {
-		fprintf(stderr, "ERROR: path too long\n");
+		error("path '%s' too long", argv[optind + 1]);
 		ret = 1;
 		goto out;
 	}
@@ -1579,8 +1539,7 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 		root = btrfs_read_fs_root(orig_root->fs_info, &key);
 		if (IS_ERR(root)) {
 			errno = -PTR_ERR(root);
-			fprintf(stderr, "fail to read root %llu: %m\n",
-					root_objectid);
+			error("failed to read root %llu: %m", root_objectid);
 			root = orig_root;
 			ret = 1;
 			goto out;
@@ -1601,7 +1560,7 @@ static int cmd_restore(const struct cmd_struct *cmd, int argc, char **argv)
 		ret = regcomp(&match_reg, match_regstr, match_cflags);
 		if (ret) {
 			regerror(ret, &match_reg, reg_err, sizeof(reg_err));
-			fprintf(stderr, "Regex compile failed: %s\n", reg_err);
+			error("regex compilation failed: %s", reg_err);
 			goto out;
 		}
 		mreg = &match_reg;
